@@ -172,6 +172,16 @@ pub struct Frames {
     pub slowest: Vec<Arc<FrameData>>,
 }
 
+impl Frames {
+    fn all_uniq(&self) -> Vec<Arc<FrameData>> {
+        let mut all = self.slowest.clone();
+        all.extend(self.recent.iter().cloned());
+        all.sort_by_key(|frame| frame.frame_index);
+        all.dedup_by_key(|frame| frame.frame_index);
+        all
+    }
+}
+
 #[derive(Clone)]
 pub struct Paused {
     /// What we are viewing
@@ -203,9 +213,12 @@ pub struct ProfilerUi {
 
 impl Default for ProfilerUi {
     fn default() -> Self {
+        let frame_view = GlobalFrameView::default();
+        frame_view.lock().set_max_recent(60 * 10); // We can't currently scroll back anyway
+
         Self {
             options: Default::default(),
-            frame_view: Default::default(),
+            frame_view,
             is_panning: false,
             is_zooming: false,
             paused: None,
@@ -377,8 +390,15 @@ impl ProfilerUi {
     pub fn ui(&mut self, ui: &Ui<'_>) {
         #![allow(clippy::collapsible_else_if)]
 
+        puffin::profile_function!();
+
+        let mut scopes_on = puffin::are_scopes_on();
+        ui.checkbox("Profiling enabled", &mut scopes_on);
+        puffin::set_scopes_on(scopes_on);
+
         if !puffin::are_scopes_on() {
-            ui.text_colored(ERROR_COLOR, "The puffin profiler is OFF!");
+            ui.same_line();
+            ui.text_colored(ERROR_COLOR, "No new scopes are being recorded!");
         }
 
         let mut hovered_frame = None;
@@ -497,6 +517,8 @@ impl ProfilerUi {
         frame: &Arc<FrameData>,
         (min_ns, max_ns): (NanoSecond, NanoSecond),
     ) -> f32 {
+        puffin::profile_function!();
+
         if self.options.canvas_width_ns <= 0.0 {
             self.options.canvas_width_ns = (max_ns - min_ns) as f32;
             self.options.zoom_to_relative_ns_range = None;
@@ -531,7 +553,10 @@ impl ProfilerUi {
             let mut paint_stream = || -> Result<()> {
                 if self.options.merge_scopes {
                     let frames = vec![frame.clone()];
-                    let merges = puffin::merge_scopes_for_thread(&frames, thread_info)?;
+                    let merges = {
+                        puffin::profile_scope!("merge_scopes");
+                        puffin::merge_scopes_for_thread(&frames, thread_info)?
+                    };
                     for merge in merges {
                         paint_merge_scope(info, &mut self.options, 0, &merge, 0, cursor_y)?;
                     }
@@ -567,10 +592,31 @@ impl ProfilerUi {
 
     /// Returns hovered, if any
     fn show_frames(&mut self, ui: &Ui<'_>) -> Option<Arc<FrameData>> {
+        puffin::profile_function!();
         let frames = self.frames();
 
         let mut hovered_frame = None;
 
+        {
+            let uniq = frames.all_uniq();
+            let bytes: usize = uniq.iter().map(|frame| frame.num_bytes).sum();
+            ui.text(format!(
+                "{} frames recorded ({:.1} MB)",
+                uniq.len(),
+                bytes as f64 * 1e-6
+            ));
+
+            let mut memory_length = self.frame_view.lock().max_recent() as u64;
+            imgui::Slider::new("Max frames to store", 10, 100_000)
+                .display_format(format!(
+                    "%d (~ {:.1} min at 60 Hz, ~ {:.0} MB)",
+                    memory_length as f32 / 60.0 / 60.0,
+                    memory_length as f32 * bytes as f32 / uniq.len() as f32 * 1e-6,
+                ))
+                .build(ui, &mut memory_length);
+            self.frame_view.lock().set_max_recent(memory_length as _);
+        }
+
         ui.columns(2, "columns", false);
         ui.set_column_width(0, 64.0);
 
